from django.test import TestCase, RequestFactory
from freezegun import freeze_time

class TestListComments(TestCase):
     def test_it(self):
         response = self.client.get("/comments/")
         self.assertEqual(response.status_code, 200)

     def test_with_request_factory(self):
         from apps import views
         request = RequestFactory()
         response = views.comment_list(request)
         self.assertEqual(response.status_code, 200)

     def test_django_create(self):
         import json
         import pytz
         from datetime import datetime
         from apps.models import Thread, Comment

         thread = Thread.objects.create(
             name="test_name",
             max_comments=3,
             publishing_time=datetime(2020, 8, 1, tzinfo=pytz.timezone("UTC")))
         Comment.objects.create(thread=thread, content="content")

         response = self.client.get("/comments/")

         self.assertEqual(response.status_code, 200)
         content = json.loads(response.content)
         self.assertEqual(len(content["threads"]), 1)

     def test_factory(self):
         import json
         from apps.tests.factories import CommentFactory
         # あらかじめ共通となるデフォルト値が設定されているため、 CommentFactory() を呼び出すだけで必要なデータが作成できる
         comment = CommentFactory()

         response = self.client.get("/comments/")

         self.assertEqual(response.status_code, 200)
         content = json.loads(response.content)
         # Thread が1件存在することc
         self.assertEqual(len(content["threads"]), 1)

     @freeze_time("2020-07-01 10:00:00")
     def test_freeze_time(self):
         import json
         import pytz
         from datetime import datetime
         from apps.models import Thread, Comment
         thread = Thread.objects.create(
             name="test_name",
             max_comments=3,
             publishing_time=datetime(2020, 8, 1, tzinfo=pytz.timezone("UTC")))
         Comment.objects.create(thread=thread, content="content")

         response = self.client.get("/comments/")

         self.assertEqual(response.status_code, 200)
         content = json.loads(response.content)
         self.assertEqual(len(content["threads"]), 0)

     def test_mock(self):
         import pytz
         from datetime import datetime
         from apps.models import Thread, Comment
         from unittest import mock
         thread = Thread.objects.create(
             name="test_name",
             max_comments=1,
             publishing_time=datetime(2020, 8, 1, tzinfo=pytz.timezone("UTC")))

         with mock.patch("django.core.mail.send_mail") as m:
             response = self.client.post("/comments/add", {
                 "thread_id": thread.id,
                 "content": "content",
             })
             self.assertEqual(m.called, True)

         self.assertEqual(response.status_code, 200)

