import logging
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.utils import timezone
from django.core import mail
from apps.models import Thread, Comment
logger = logging.getLogger(__name__)

def comment_list(request):
    """ コメント一覧API
    """
    # レスポンス用の辞書を用意する
    data = {"threads": []}
    # スレッドを全件取得する
    threads = Thread.objects.all()
    # スレッドごとにループをまわす
    for thread in threads:
        # 未来日の場合はスキップ
        if thread.publishing_time > timezone.now():
            logging.warning("未来日が含まれています:: スレッドID: %d, 公開日時: %s", thread.id, thread.publishing_time)
            continue
        # スレッドの名前とスレッドのコメントを追加する
        data["threads"].append({
            "name": thread.name,
            "comments": [comment.content for comment in thread.comment_set.all()]
        })
    # JSON としてレスポンスを返す
    return JsonResponse(data, json_dumps_params={'ensure_ascii': False})

@csrf_exempt
def comment_add(request):
    """ コメント追加API
    """
    # スレッドの取得
    thread = get_object_or_404(Thread, id=request.POST["thread_id"])
    # スレッドがコメント数制限に達した場合、コメントを追加できない
    if Comment.objects.filter(thread=thread).count() == thread.max_comments:
        return JsonResponse({
            "error": "コメント制限に達しました"
        }, json_dumps_params={'ensure_ascii': False})
    # コメントの作成
    Comment.objects.create(thread=thread, content=request.POST["content"],)
    # スレッドのコメント数制限に達した場合、管理者に通知メールを送信する
    if Comment.objects.filter(thread=thread).count() == thread.max_comments:
        mail.send_mail(subject="コメント通知メール",
                       message=f"{thread.name}のコメント数が上限に達しました",
                       from_email="admin@localhost",
                       recipient_list=["admin@localhost"],
                       fail_silently=False,)
    return JsonResponse({})
